# Copyright (c) SenseTime. All Rights Reserved.
'''
for multi-gpu testing
'''

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import argparse

import torch
import subprocess
import numpy as np


parser = argparse.ArgumentParser(description='siamrpn tracking')
parser.add_argument('--dataset', type=str,
                    help='datasets')
parser.add_argument('--config', default='', type=str,
                    help='config file')
parser.add_argument('--start_epoch', type=int, default=5)
parser.add_argument('--end_epoch', type=int, default=20)
args = parser.parse_args()


def multi_process():
    gpu_num = torch.cuda.device_count()
    gpu_epoch = [[] for _ in range(gpu_num)]
    gpu_load = np.zeros(gpu_num, dtype=np.int)
    epochs = list(range(args.start_epoch, args.end_epoch+1))
    for epoch in epochs:
        # allocate work load for each GPU
        gpu_idx = np.argmin(gpu_load)
        gpu_epoch[gpu_idx].append(epoch)
        gpu_load[gpu_idx] += 1

    # launch multi-process
    processes = list()
    for gpu_idx in range(gpu_num):
        args_epochs = gpu_epoch[gpu_idx]
        args_epochs = list(map(lambda s: str(s), args_epochs))
        args_epochs = ','.join(args_epochs)
        cmd = [
               'python',
               '../../tools/test.py',
               '--dataset={}'.format(args.dataset),
               '--config={}'.format(args.config),
               '--epochs={}'.format(args_epochs),
               '--gpu_idx={}'.format(gpu_idx)]
        process = subprocess.Popen(cmd)
        processes.append(process)
    for process in processes:
        process.wait()
    print('Test Done.')


if __name__ == '__main__':
    multi_process()